import { Determinants } from "./determinants/Determinants";
import { ConnectionStatesEnum } from "./enum/ConnectionStates";
import { RoleUserEnum } from "./enum/RoleUserEnum";
import { Resources } from "./resources/Resources";
var Context = /** @class */ (function () {
    function Context(config) {
        this.resources = new Resources();
        this.determinants = new Determinants({
            Role: RoleUserEnum.Host,
            Updating: false,
            ConnectionState: { curState: ConnectionStatesEnum.DISCONNECTED, revState: ConnectionStatesEnum.DISCONNECTED, reason: undefined },
            HasAudio: true,
            HasCamera: false,
            HasScreen: false,
            Muted: false,
            Init: false
        });
        if (config.AppId)
            this.getDeterminants().setAppId(config.AppId);
        if (config.Uid)
            this.getDeterminants().setUid(config.Uid);
    }
    Context.prototype.getDeterminants = function () {
        return this.determinants;
    };
    Context.prototype.setDeterminants = function (value) {
        this.determinants = value;
    };
    Context.prototype.getResources = function () {
        return this.resources;
    };
    Context.prototype.setResources = function (value) {
        this.resources = value;
    };
    Context.prototype.canInit = function () {
        if (this.getDeterminants().getInit()) {
            return false;
        }
        return true;
    };
    Context.prototype.canRegionChange = function () {
        if (this.getDeterminants().getConnectionState().curState != ConnectionStatesEnum.DISCONNECTED ||
            !this.getResources().getClient()) {
            return false;
        }
        return true;
    };
    Context.prototype.canActiveProxy = function () {
        if (this.getDeterminants().getConnectionState().curState != ConnectionStatesEnum.DISCONNECTED ||
            !this.getResources().getClient()) {
            return false;
        }
        return true;
    };
    Context.prototype.canDisconnect = function () {
        if (!this.getDeterminants().getInit() ||
            !this.getResources().getClient()) {
            return false;
        }
        return true;
    };
    Context.prototype.canMute = function () {
        if (this.getDeterminants().getInit()) {
            return false;
        }
        return true;
    };
    Context.prototype.canShareCamera = function () {
        if (this.getDeterminants().getConnectionState().curState != ConnectionStatesEnum.CONNECTED)
            return false;
        if (!this.getDeterminants().getInit())
            return false;
        if (this.getDeterminants().getHasScreen())
            return false;
        return true;
    };
    Context.prototype.canShareScreen = function () {
        if (this.getDeterminants().getConnectionState().curState != ConnectionStatesEnum.CONNECTED)
            return false;
        if (!this.getDeterminants().getInit())
            return false;
        if (this.getDeterminants().getHasCamera())
            return false;
        return true;
    };
    Context.prototype.canChangeRole = function () {
        if (this.getDeterminants().getConnectionState().curState != ConnectionStatesEnum.DISCONNECTED ||
            !this.getResources().getClient()) {
            return false;
        }
        return true;
    };
    return Context;
}());
export { Context };
