import { EErrorEngineTypes } from "./enum/EErrorEngineTypes";
import { EMessageType } from "./enum/EMessageType";
var Logger = /** @class */ (function () {
    function Logger(engines) {
        this.debug = false;
        this.engines = [EErrorEngineTypes.CONSOLE, EErrorEngineTypes.VIEW];
        this.engines = engines;
    }
    Logger.prototype.error = function (message) {
        message.type = EMessageType.ERROR;
        this.doLog(message);
    };
    Logger.prototype.info = function (message) {
        message.type = EMessageType.INFO;
        this.doLog(message);
    };
    Logger.prototype.warning = function (message) {
        message.type = EMessageType.WARNING;
        this.doLog(message);
    };
    Logger.prototype.log = function (message) {
        switch (message.type) {
            case EMessageType.ERROR:
                this.error(message);
                break;
            case EMessageType.INFO:
                this.info(message);
                break;
            case EMessageType.WARNING:
                this.warning(message);
                break;
            default:
                this.info(message);
                break;
        }
    };
    Logger.prototype.doLog = function (message) {
        var _this = this;
        this.engines.forEach(function (engine) {
            switch (engine) {
                case EErrorEngineTypes.CONSOLE:
                    _this.doConsoleLog(message);
                    break;
                case EErrorEngineTypes.SENTRY:
                    //TODO:integration with sentry
                    break;
                case EErrorEngineTypes.VIEW:
                    _this.doNotifyView(message);
                    break;
                default:
                    break;
            }
        });
    };
    Logger.prototype.doConsoleLog = function (message) {
        switch (message.type) {
            case EMessageType.ERROR:
                this.colorLog(message, "error");
                break;
            case EMessageType.INFO:
                this.colorLog(message, "info");
                break;
            case EMessageType.WARNING:
                this.colorLog(message, "warning");
                break;
            default:
                break;
        }
    };
    Logger.prototype.colorLog = function (message, color) {
        color = color || "black";
        switch (color) {
            case "success":
                color = "Green";
                break;
            case "info":
                color = "DodgerBlue";
                break;
            case "error":
                color = "Red";
                break;
            case "warning":
                color = "Orange";
                break;
            default:
                color = color;
        }
        console.log("%c" + "[AgoraModule] - " + message.body, "color:" + color, message);
    };
    Logger.prototype.doNotifyView = function (message) {
        if (this.callbackMessage) {
            this.callbackMessage(message);
        }
    };
    Logger.prototype.getCallbackMessage = function () {
        return this.callbackMessage;
    };
    Logger.prototype.setCallbackMessage = function (value) {
        this.callbackMessage = value;
    };
    return Logger;
}());
export { Logger };
