import { CustomError } from "../../exception/CustomError";
import Validator from 'validatorjs';
var BaseRule = /** @class */ (function () {
    function BaseRule(params) {
        this.errors = [];
        this.passes = false;
        this.fails = false;
        this.params = params;
    }
    BaseRule.prototype.addError = function (error) {
        var _this = this;
        var customError = CustomError.parseCustomError(error);
        if (customError instanceof Array) {
            customError.map(function (itemError) { _this.errors.push(itemError); });
        }
        else {
            this.errors.push(customError);
        }
    };
    BaseRule.prototype.getErrors = function () {
        return this.errors;
    };
    BaseRule.prototype.evaluate = function (data, rules, messages) {
        this.validator = new Validator(data, rules, messages);
        var passes = this.validator.passes();
        var fails = this.validator.fails();
        this.addError(this.validator.errors);
        if (passes)
            return true;
        else if (fails)
            return false;
        return false;
    };
    BaseRule.prototype.reset = function () {
        this.errors = [];
        this.passes = false;
        this.fails = false;
    };
    BaseRule.prototype.passesOrFails = function (passes) {
        if (passes.every(function (pass) { return pass == true; })) {
            this.passes = true;
            return true;
        }
        else {
            this.fails = true;
            return false;
        }
    };
    BaseRule.prototype.validate = function (rules, context) {
        return this.passes;
    };
    return BaseRule;
}());
export { BaseRule };
