import { InitAction } from "../action/InitAction";
import { Context } from "../context/Context";
import { ERuleTypes } from "../rules/enum/ERuleTypes";
import { IManager } from "./interface/IManager";
import { v4 as uuidv4 } from "uuid";
import { EActionStatus } from "../action/enum/EActionStatus";
import { EMessageType } from "../logger/enum/EMessageType";
import { EOperationStatus } from "../operation/enum/EOperationStatus";
import { BaseManager } from "./base/BaseManager";
import { CustomError } from "../exception/CustomError";
import { ChangeModuleConfigParams } from "../action/type/ChangeModuleConfigParams";
import { ChangeModuleConfigRules } from "../rules/ChangeModuleConfigRules";
import { ChangeModuleConfigOperation } from "../operation/ChangeModuleConfigOperation";
const parseError = require('parse-error');

export class ChangeModuleConfigManager extends BaseManager implements IManager {

    constructor(action : InitAction, context: Context){
        super(action, context);
    }

    /*@overwrite*/
    public async performAction(): Promise<EActionStatus> {

        return new Promise( async (resolve, reject)=>{

            let params : ChangeModuleConfigParams = Object.assign(this.getAction().getParams()); 

            let validationRules = new ChangeModuleConfigRules(params); 

            validationRules.validate([ERuleTypes.IS_VALID_CONFIG_PARAMS], this.context);
        
            if(validationRules.passes){

                try {
                    let changeModuleConfigOperation  = new ChangeModuleConfigOperation(this.context);
                        await changeModuleConfigOperation.handler(params);
    
                        if(changeModuleConfigOperation.status == EOperationStatus.PERFORMED){
            
                            this.action.status = EActionStatus.PERFORMED;
            
                            this.logger.log({
                                id: uuidv4(),
                                title: this.action.type,
                                body: "Action "+this.action.type + " performaced",
                                action: this.action,
                                errors: validationRules.errors,
                                type:EMessageType.INFO
                            });

                        }else{

                            let errors : Array<CustomError> = [];
                            if(changeModuleConfigOperation.errors) errors = [...errors, ...changeModuleConfigOperation.errors];

                            this.action.status = EActionStatus.FAILED;
                            this.logger.log({
                                id: uuidv4(),
                                title: this.action.type,
                                body: "Something operations not performaced in action "+this.action.type,
                                action: this.action,
                                errors: errors,
                                type: EMessageType.ERROR
                            });

                        }

                } catch (error) {

                    let errors = parseError(error);

                    this.action.status = EActionStatus.FAILED;
                    
                    this.logger.log({
                        id: uuidv4(),
                        title: this.action.type,
                        body: "Something operations not performaced in action "+this.action.type,
                        action: this.action,
                        errors: errors,
                        type: EMessageType.ERROR
                    });
                }
                
            }else{
                
                this.action.status = EActionStatus.FAILED;
                      
                this.logger.log({
                    id: uuidv4(),
                    title: this.action.type,
                    body: "Action "+this.action.type + " not performaced",
                    action: this.action,
                    errors: validationRules.errors,
                    type: EMessageType.ERROR
                });
            }

            resolve(this.action.status);

        });

    }

}