import { Context } from "../context/Context";
import { BaseOperation } from "./base/BaseOperation";
import { EOperationStatus } from "./enum/EOperationStatus";
import { EOperationType } from "./enum/EOperationType";
import { IOperation } from "./interface/IOperation";
import { ActionParams } from "../action/type/ActionParams";
import { ActiveProxyActionParams } from "../action/type/ActiveProxyActionParams";

 
export class ActiveProxyOperation extends BaseOperation implements IOperation {

    constructor(context: Context){
        super(context);
        this.type=EOperationType.ACTIVE_PROXY;
    }

    /*@overwrite*/
    public async handler(params ? : ActionParams) : Promise<string>{

        return new Promise( async (resolve, reject)=>{

            try {

                this.context.getDeterminants().setUpdating(true);

                let paramsObj : ActiveProxyActionParams = Object.assign(params ? params : {});
                let client = this.context.getResources().getClient();
                
                if(paramsObj.active){
                    await client.startProxyServer(paramsObj.type);
                    this.context.getDeterminants().setHasProxy(true);
                }else{
                    await client.stopProxyServer();
                    this.context.getDeterminants().setHasProxy(false);
                }
                this.status = EOperationStatus.PERFORMED;

            } catch (error) {

                this.addError(error);
                this.status = EOperationStatus.FAILED;
                
            }

            resolve(this.status);
        
        });

    }

   
}