import AgoraRTC, { ILocalAudioTrack, ILocalVideoTrack } from "agora-rtc-sdk-ng";
import { ActionParams } from "../action/type/ActionParams";
import { RegionChangeActionParams } from "../action/type/RegionChangeActionParams";
import { ToggleVoiceActionParams } from "../action/type/ToggleVoiceActionParams";
import { Context } from "../context/Context";
import { CustomError } from "../exception/CustomError";
import { ErrorType } from "../exception/enum/ErrorType";
import { BaseOperation } from "./base/BaseOperation";
import { EOperationStatus } from "./enum/EOperationStatus";
import { EOperationType } from "./enum/EOperationType";
import { IOperation } from "./interface/IOperation";
const parseError = require('parse-error');

export class MuteAudioOperation extends BaseOperation implements IOperation {

    constructor(context: Context){
        super(context);
        this.type=EOperationType.MUTE_AUDIO;
    }

    /*@overwrite*/
    public async handler(params ? : ActionParams) : Promise<string>{

        return new Promise( async (resolve, reject)=>{

            try {

                this.context.getDeterminants().setUpdating(true);

                let toggleVoiceParams : ToggleVoiceActionParams = Object.assign(params ? params : {});

                let track = this.context.getResources().getAudioTrack();
                    if(track && track.enabled) track.setVolume(toggleVoiceParams.action == true ? 0 : 100);

                this.context.getDeterminants().setMuted(toggleVoiceParams.action);
                
                this.status = EOperationStatus.PERFORMED;

            } catch (error) {

                this.addError(error);
                this.status = EOperationStatus.FAILED;
                
            }

            resolve(this.status);
        
        });

    }

   
}