import { ActionParams } from "../action/type/ActionParams";
import { UnsubscribeActionParams } from "../action/type/UnsubscribeActionParams";
import { Context } from "../context/Context";
import { BaseOperation } from "./base/BaseOperation";
import { EOperationStatus } from "./enum/EOperationStatus";
import { EOperationType } from "./enum/EOperationType";
import { IOperation } from "./interface/IOperation";
const parseError = require('parse-error');

export class UnsubscribeOperation extends BaseOperation implements IOperation {

    constructor(context: Context){
        super(context);
        this.type=EOperationType.UNSUBSCRIBE;
    }

    /*@overwrite*/
    public async handler(params ? : ActionParams) : Promise<string>{

        return new Promise( async (resolve, reject)=>{

            try {

                this.context.getDeterminants().setUpdating(true);
                
                let client = this.context.getResources().getClient();

                let unsubscribeActionParams : UnsubscribeActionParams = Object.assign(params ? params : {});

                let userExist = false; 

                client.remoteUsers.map((user)=>{
                    if(user.uid == unsubscribeActionParams.remoteUser.uid){
                        userExist = true;
                    }
                });
                
                if(userExist){
                    await client.unsubscribe(unsubscribeActionParams.remoteUser, unsubscribeActionParams.mediaType);
                }

                this.status = EOperationStatus.PERFORMED;

            } catch (error) {
                this.addError(error);
                this.status = EOperationStatus.FAILED;
            }

            resolve(this.status);
        
        });

    }

   
}