import { Context } from "../context/Context";
import { IRule } from "./interface/IRule";

/*
* use https://www.npmjs.com/package/validatorjs
*/
import { ERuleTypes } from "./enum/ERuleTypes";
import { ActionParams } from "../action/type/ActionParams";
import { BaseRule } from "./base/BaseRule";
import { ActiveProxyActionParams } from "../action/type/ActiveProxyActionParams";
import { CommonRules } from "./CommonRules";


export class ActiveProxyRules extends BaseRule implements IRule {

    constructor(params: ActionParams){
        super(params);
    }

    /*@overwrite*/
    public validate(rules: Array<string>, context: Context): boolean {

        let passes : any = [];

        rules.forEach(rule => {
            switch (rule) {
                case ERuleTypes.PROXY_VALID_TYPE:
                    passes.push(this.validProxyType(this.params));
                break;
                case ERuleTypes.IS_CLIENT_INIT:
                    passes.push(new CommonRules(this.params).isClientInit(context, this));
                break;
            }
        });

        if(passes.every((pass : boolean) => pass == true)){
            this.passes = true;
            return true;
        }else{
            this.fails = true;
            return false;
        }
       
    }

    public validProxyType(params: ActionParams): boolean {

        let paramsObj: ActiveProxyActionParams = Object.assign(params);

        let data = {
            proxy_type: paramsObj.type,
        };

        let rules = {
            proxy_type: 'numeric|in:3,5'
        };

        let messages = {
            "number.proxy_type":'Invalid proxy type',
            "in.proxy_type":'Invalid proxy type'
        };

        return this.evaluate(data, rules, messages);
    }

}