import { Context } from "../context/Context";
import { IRule } from "./interface/IRule";

/*
* use https://www.npmjs.com/package/validatorjs
*/
import Validator from 'validatorjs';
import { ERuleTypes } from "./enum/ERuleTypes";
import { ActionParams } from "../action/type/ActionParams";
import { InitActionParams } from "../action/type/InitActionParams";
import { BaseRule } from "./base/BaseRule";
import { CommonRules } from "./CommonRules";


export class InitRules extends BaseRule implements IRule {

    constructor(params: ActionParams){
        super(params);
    }

    /*@overwrite*/
    public validate(rules: Array<string>, context: Context): boolean {

        let passes : any = [];

        rules.forEach(rule => {
            switch (rule) {
                case ERuleTypes.IS_INITIATED:
                    passes.push(new CommonRules(this.params).isInited(context, this));
                break;

                case ERuleTypes.IS_NOT_INITIATED:
                    passes.push(new CommonRules(this.params).isNotInited(context, this));
                break;

                case ERuleTypes.INIT_PARAM_IS_TRUE:
                    passes.push(this.initParamIsTrue(this.params));
                break;
            }
        });

        if(passes.every((pass : boolean) => pass == true)){
            this.passes = true;
            return true;
        }else{
            this.fails = true;
            return false;
        }
       
    }

    public initParamIsTrue(params: ActionParams): boolean {

        let paramsObj: InitActionParams = Object.assign(params);
        
        let data = {
            init_param: paramsObj.init
        };

        let rules = {
            init_param: 'boolean|accepted'
        };

        let messages = {
            "accepted.init_param":'Init param must be true to init agora'
        };

        return this.evaluate(data, rules, messages);
    }

}